package com.winhc.bigdata.spark.ng.dynamic.utils

import com.winhc.bigdata.spark.implicits.Bool._
import com.winhc.bigdata.spark.ng.dynamic.utils.DynamicDimConfiguration.DDC
import org.apache.commons.lang3.StringUtils
import org.json4s._
import org.json4s.jackson.JsonMethods._
import org.json4s.jackson.Serialization

/**
 * @author ZhangJi
 * @since 2021-07-08 10:51
 */
object DynamicFiledUtils {
  val SIMPLE_FILED_CONFIG: Map[String, Map[String, Map[String, Any]]] = DDC.filter(e => e._2.getOrElse("sample", false).asInstanceOf[Boolean]).map(e => (e._1, e._2("list_field").asInstanceOf[Map[String, Map[String, Any]]]))

  def createSimpleDynamicInfo(tn: String, data: Map[String, String], update_type: String = "insert"): (String, Map[String, Any]) = {
    val config = SIMPLE_FILED_CONFIG.getOrElse(tn, Map.empty)
    val filter_set = config.filter(e => e._2.getOrElse("when", "insert").equals(update_type)).map(e => e._2.getOrElse("from", e._1)).toSet
    (DDC(tn)("code").asInstanceOf[Map[String, String]].getOrElse(update_type, null),
      data.filterKeys(e => filter_set.contains(e)))
  }

  def distinctBy[A, B](xs: Seq[A])(f: A => B): List[A] =
    scala.reflect.internal.util.Collections.distinctBy(xs.toList)(f)

  def createDynamicInfo(tn: String, data: Map[String, String]): (String, Map[String, Any]) = {
    val lf = DDC(tn)("list_field").asInstanceOf[Map[String, Map[String, Any]]]
    val dynamicInfo = lf.map(elem => {
      val from = elem._2.getOrElse("from", Seq(elem._1)).asInstanceOf[Seq[String]]
      elem._2.get("type") match {
        case Some("array") => {
          (elem._1, distinctBy(from.flatMap(n => {
            val v = data.getOrElse(n, null)
            if (StringUtils.isEmpty(v)) {
              List.empty[JValue]
            } else {
              var array = parse(v, useBigDecimalForDouble = true).asInstanceOf[JArray]
              array = (array == null) ? new JArray(List.empty[JValue]) | array
              array.arr
            }
          }))(jo => jo \ "name"))
        }
        case Some("object") => {
          (elem._1, from.map(e => {
            val v = data.getOrElse(e, null)
            if (StringUtils.isEmpty(v)) {
              (e.replace("_info", ""), null)
            } else
              (e.replace("_info", ""), parse(data.getOrElse(e, null)))
          }).toMap)
        }
        case Some("json") => {
          val v = data.getOrElse(from.head, null)
          if (StringUtils.isEmpty(v)) {
            (elem._1, null)
          } else {
            (elem._1, parse(v))
          }
        }
        case _ => (elem._1, data.getOrElse(from.head, null))
      }
    })
    (DDC(tn)("code").asInstanceOf[Map[String, String]]("insert"), dynamicInfo)
  }


  def main(args: Array[String]): Unit = {
    val str =
      """
        |
        |{"area":"6.5867","create_time":"2020-06-02 23:40:06","deleted":"0","ds":"20201227","end_date":"2020-05-20 00:00:00","evaluate_amount":"2568.8130","land_aministrative_area":"440705000","land_area":"6.5867","land_loc":"崖门镇洞南村委会沙荞(土名)","land_mark":"","land_num":"0917000099","mortgage_amount":"2568.8130","mortgagee":"江门新会农村信商业银行股份有限公司崖门支行","mortgagee_company_id":null,"mortgagor":"江门市新会区崖门镇桥盛纸厂","mortgagor_company_id":"960d34a5d353bc15d25666d6316da0aa","nature":"个人独资企业","other_item_num":"新他项(2012)第00310号","rowkey":"0045f1d8d2d93979f26b8ae91cef0d6e","source_url":"http://www.landchina.com/default.aspx?tabid=386&comname=default&wmguid=aeb2d2f3-7519-4112-add8-7ba77dc33803&recorderguid=2ab04c53-fe0f-4ded-ba60-5b42a9eeb53d","start_date":"2012-06-27 00:00:00","update_time":"2020-06-02 23:40:06","use_for":"工业用地","use_right_num":"新国用(2012)第01719号","use_type":"国有出让"}
        |
        |""".stripMargin
    implicit val formats: AnyRef with Formats = Serialization.formats(NoTypeHints)
    val value: Map[String, String] = parse(str).extract[Map[String, String]]
    val str2 =
      """
        |{
        |    "PLAN_DATE": "",
        |    "DEPARTMENT": "",
        |    "PLAINTIFF": "郭宝录",
        |    "ROWKEY": "d4bc5a9ba809c80c6271bee66b350d4e",
        |    "START_DATE": "2019-11-26 09:00:00",
        |    "UPDATE_TIME": "2020-09-10 05:15:03",
        |    "DELETED": "0",
        |    "FLAG": "",
        |    "DEFENDANT": "宋斌",
        |    "JUDGE": "",
        |    "CASE_NO": "（2019）青0103民初3440号",
        |    "LITIGANT": "宋斌,郭宝录",
        |    "LITIGANT_INFO": "[{\"name\":\"宋斌\",\"litigant_id\":\"\"},{\"name\":\"郭宝录\",\"litigant_id\":\"\"}]",
        |    "AREA": "",
        |    "PLAINTIFF_INFO": "[{\"name\":\"郭宝录\",\"litigant_id\":\"\"}]",
        |    "SOURCE": "法海接口 c2019630103minchu3440_t20191126",
        |    "CASE_REASON": "民间借贷纠纷",
        |    "rowkey": "d4bc5a9ba809c80c6271bee66b350d4e",
        |    "DEFENDANT_INFO": "[{\"name\":\"宋斌\",\"litigant_id\":\"\"}]",
        |    "CONTENT": "{\"当事人\":\"我院定于 2019 年11月26日 09时00分在本院第十一法庭依法公开审理郭宝录诉宋斌民间借贷纠纷一案。\",\"发布时间\":1572019200000,\"案号\":\"（2019）青0103民初3440号\",\"法院\":\"西宁市城中区人民法院\",\"开庭时间\":1571932800000}",
        |    "CREATE_TIME": "2020-09-10 05:15:03",
        |    "COURT_ROOM": "第十一法庭",
        |    "COURT": "青海省西宁市城中区人民法院"
        |  }
        |""".stripMargin
    val value2: Map[String, String] = parse(str2.toLowerCase()).extract[Map[String, String]]
    import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
    //    val tuple = createDynamicInfo("company_equity_info", value)
    val tuple = createDynamicInfo("company_court_open_announcement", value2)
    println(tuple._2.toJson())

  }
}
